/**
 * A custom Quill highlighting module for markup modes/styles.  It manages
 * multiple mode instances  that are used to apply text formatting to the
 * contents of the control.
 *
 * The module is added to the component per the API instructions:
 * https://quilljs.com/guides/building-a-custom-module/
 *
 * Once the module is initialized the highlighting mode can be changed with
 * the `set` functions.  This just changes what the reference internally to
 * another class that handles the formatting.
 *
 * #### Examples:
 *
 * ```javascript
 * import {MarkupMode, Markup} from './lib/markup';
 *
 * Quill.register('modules/markup', Markup);
 * const quill = new Quill('#editor', {
 *     modules: {
 *         history: {
 *             delay: 2000,
 *             maxStack: 500,
 *             userOnly: true
 *         },
 *         markup: {
 *             mode: MarkupMode.text,
 *             custom: {
 *                 foreground: 'black',
 *                 background: 'white',
 *             }
 *         },
 *         toolbar: false
 *     },
 *     theme: 'snow'
 * });
 * ...
 * const hl = quill.getModule('highlight');
 * hl.set({
 *     content: 'some value',
 *     mode: MarkupMode.markdown
 * });
 * ```
 *
 * Note that initial content cannot be set in the contructor to the module.
 * it is overwritten with the contents of the `#editor` div as Qill is
 * instatiated.  It can be set after creation of the Quill instance using
 * the `.setContent('')` function.
 *
 * @module Markup
 */
'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var util_fontlist_1 = require("util.fontlist");
var util_section_1 = require("util.section");
var helpers_1 = require("./helpers");
var highlights_1 = require("./highlights");
var modes_1 = require("./modes");
var nilEvent = function nilEvent(val) {
    val = null;
};
var MarkupMode;
(function (MarkupMode) {
    MarkupMode[MarkupMode["markdown"] = 0] = "markdown";
    MarkupMode[MarkupMode["text"] = 1] = "text";
})(MarkupMode = exports.MarkupMode || (exports.MarkupMode = {}));
var defaultOptions = {
    content: '',
    custom: {},
    dirtyLimit: 20,
    fontName: 'Fira Code',
    fontSize: 12,
    followLinks: false,
    highlight: 'solarized-light',
    idleDelay: 2000,
    mode: MarkupMode.text,
    onChange: nilEvent,
    onClick: nilEvent,
    onClickLink: nilEvent
};
require('./styles.css');
var debug = require('debug')('markup');
var pkg = require('../package.json');

var Markup = function () {
    function Markup(quill) {
        var _this = this;

        var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        _classCallCheck(this, Markup);

        // As the document is modified the number of characters that are changed
        // is counted.  When a "dirty" limit is reached and the user is idle then
        // a full rescan of the document block elements will occur.  The smaller
        // the limit, the sooner this will occur when idle.
        this._dirtyIdle = false;
        this._dirtyInline = false;
        this._dirtyCount = 10;
        // The time between each incremental section scan
        this._delay = 250;
        this._changing = false;
        // The time before idle detection
        this._idle = true;
        this._idleDelay = 2000;
        this._fonts = ['Fira Code', 'Inconsolata', 'Source Code Pro'];
        this._line = {
            start: 0,
            end: 0,
            text: ''
        };
        this._modes = {};
        this._paste = false;
        debug('Initializing markup module');
        this._quill = quill;
        this._opts = Object.assign({}, defaultOptions, opts);
        this._editor = quill.container;
        debug('quill (local): %o', quill);
        debug('Quill (global): %o', helpers_1.Quill);
        debug('Editor id: %s', this.editorKey);
        this._modes[MarkupMode.markdown] = new modes_1.Markdown(quill);
        this._modes[MarkupMode.text] = new modes_1.Text(quill);
        debug('modes: %O', this._modes);
        this._fonts = lodash_1.union(this._fonts, util_fontlist_1.getFontList());
        debug('available fonts: %O', this.fonts);
        // bind all potential callbacks to the class.
        ['handleClick', 'handleEditorChange', 'handlePaste', 'handleTextChange', 'resetInactivityTimer', 'markIdle', 'redo', 'set', 'setBold', 'setContent', 'setItalic', 'setFont', 'setFontSize', 'setHeader', 'setMode', 'setStrikeThrough', 'setUnderline', 'undo'].forEach(function (fn) {
            _this[fn] = _this[fn].bind(_this);
        });
        quill.on('editor-change', this.handleEditorChange);
        quill.on('text-change', this.handleTextChange);
        this._editor.addEventListener('paste', this.handlePaste);
        this._editor.addEventListener('click', this.handleClick);
        window.addEventListener('load', this.resetInactivityTimer);
        document.addEventListener('mousemove', this.resetInactivityTimer);
        document.addEventListener('click', this.resetInactivityTimer);
        document.addEventListener('keydown', this.resetInactivityTimer);
        this.set(opts);
    }

    _createClass(Markup, [{
        key: "resetInactivityTimer",

        /**
         * Resets the idle timer when the user does something in the app.
         */
        value: function resetInactivityTimer() {
            this._idle = false;
            clearTimeout(this._idleTimer);
            this._idleTimer = setTimeout(this.markIdle, this._idleDelay);
        }
        /**
         * When the user is idle for N seconds this function is called by a global
         * timer to set a flag and rescan the full document.  This can be an
         * expensive operation and we need to find the tradeoff limit
         */

    }, {
        key: "markIdle",
        value: function markIdle() {
            this._idle = true;
            if (this._dirtyIdle) {
                this._dirtyIdle = false;
                this._dirtyCount = 0;
                this._processor.refreshBlock();
            }
            if (this._dirtyInline) {
                this._processor.refreshInline();
                this._dirtyInline = false;
            }
        }
        /**
         * Calls the quill history redo function
         */

    }, {
        key: "redo",
        value: function redo() {
            if (this._quill.history) {
                this._quill.history.redo();
            }
        }
        /**
         * Rescans the entire document for highlighting.  This is a wrapper around
         * the processor's full refresh function.
         */

    }, {
        key: "refresh",
        value: function refresh() {
            this._processor.refreshFull();
            this._dirtyIdle = false;
            this._dirtyCount = 0;
            this._dirtyInline = false;
        }
        /**
         * Changes the current highlighting mode and display style.  It also sets
         * the content within control.
         * @param opts {HighlightOptions} configuration options.  `mode` sets the
         * editing mode.  `styling` sets the current display theme.  `custom` is
         * an object that contains custom format colors (for highlights).  The
         * `content` resets the content within the control.  If this is null it
         * is ignored.
         */

    }, {
        key: "set",
        value: function set(opts) {
            this._opts = Object.assign({}, defaultOptions, this._opts, opts);
            debug('options: %o', this._opts);
            this._processor = this._modes[this._opts.mode];
            debug('using processor: %o', this._processor);
            this._dirtyLimit = this._opts.dirtyLimit;
            this._idleDelay = this._opts.idleDelay;
            this._processor.style = Object.assign({
                foreground: 'black',
                background: 'white'
            }, this._opts.mode === MarkupMode.text ? {} : require('./highlighting.json'), this._opts.custom);
            debug('current highlighting colors: %o', this._processor.style);
            this.setContent(this._opts.content);
            this.setFont(this._opts.fontName);
            this.setFontSize(this._opts.fontSize);
            this.setHighlight(this._opts.highlight);
            this._editor.style['color'] = this._processor.style.foreground;
            this._editor.style['background-color'] = this._processor.style.background;
            this._line = util_section_1.line(this._opts.content, 0);
            this._processor.refreshFull();
        }
        /**
         * Will call the current processor's bold function to make the current
         * highlight or word bold.
         */

    }, {
        key: "setBold",
        value: function setBold() {
            this._processor.handleBold();
        }
        /**
         * Will update the current content of the control with the given
         * @param content {string} the new content settting for the editor.
         */

    }, {
        key: "setContent",
        value: function setContent(content) {
            if (!content && pkg['debug']) {
                // Temporary code below.  This is just convenience code for
                // testing.
                content = '*Hello World* \n';
                for (var i = 0; i < 25; i++) {
                    for (var j = 0; j < 40; j++) {
                        content += String.fromCharCode(i + 97) + " ";
                    }
                    content += '\n';
                }
            }
            this._opts.content = content;
            this._processor.text = content;
        }
        /**
         * Changes the current overall font for the editor.  This control works with
         * mono fonts, so this will set it for all text.  The fonts are all defined
         * in `./lib/fonts`.
         * @param fontName {string} the name of the font to set.
         */

    }, {
        key: "setFont",
        value: function setFont(fontName) {
            if (!this._fonts.includes(fontName)) {
                fontName = this._fonts[0];
            }
            debug('setting font: %s', fontName);
            this._opts.fontName = fontName;
            this._editor.style.fontFamily = fontName;
        }
        /**
         * Changes the current overall size of the fonts within the editor.
         * @param fontSize {number} the number of pixels in the font sizing.  This
         * will resolve to `##px`.
         */

    }, {
        key: "setFontSize",
        value: function setFontSize(fontSize) {
            debug('setting font size: %spx', fontSize);
            this._opts.fontSize = fontSize;
            this._editor.style['font-size'] = fontSize + "px";
        }
        /**
         * Calls the processor's current header creation function
         * @param level {string} the level selected by the user
         */

    }, {
        key: "setHeader",
        value: function setHeader(level) {
            this._processor.handleHeader(Number(level));
        }
    }, {
        key: "setHighlight",
        value: function setHighlight(name) {
            debug('setting highlight: %s', name);
            if (name in highlights_1.cssHighlights) {
                var oldlink = document.getElementById('highlights');
                var newlink = document.createElement('link');
                newlink.setAttribute('id', 'highlights');
                newlink.setAttribute('rel', 'stylesheet');
                newlink.setAttribute('type', 'text/css');
                newlink.setAttribute('href', highlights_1.cssHighlights[name]);
                var head = document.getElementsByTagName('head').item(0);
                if (oldlink) {
                    head.replaceChild(newlink, oldlink);
                } else {
                    head.appendChild(newlink);
                }
            }
        }
        /**
         * Calls the current processor's italic function to make the current
         * highlight or word italic.
         */

    }, {
        key: "setItalic",
        value: function setItalic() {
            this._processor.handleItalic();
        }
        /**
         * Changes the current display mode for the markup processor
         * @param mode {string} the name of the mode to set
         */

    }, {
        key: "setMode",
        value: function setMode(mode) {
            debug('setting mode: %s', mode);
            if (!(mode in MarkupMode)) {
                mode = 'text';
            }
            this.set({
                content: this._processor.text,
                mode: MarkupMode[mode]
            });
        }
        /**
         * Calls the current processor's strikthrogh function to highlight the
         * current word/selection.
         */

    }, {
        key: "setStrikeThrough",
        value: function setStrikeThrough() {
            this._processor.handleStrikeThrough();
        }
        /**
         * Calls the current processor's underline function to highlight the
         * current word/selection.
         */

    }, {
        key: "setUnderline",
        value: function setUnderline() {
            this._processor.handleUnderline();
        }
        /**
         * Calls the quill history undo function
         */

    }, {
        key: "undo",
        value: function undo() {
            if (this._quill.history) {
                this._quill.history.undo();
            }
        }
        /**
         * This event is invoked whenever the mouse is clicked within the editor.
         * It will call the user give onClick handler and pass the position within
         * the editor that was clicked.
         *
         * If the `followLinks` configuration option is given, then another handler
         * is called that will check if the clicked position is a link.  If it is,
         * then the regex match object that found this link is returned to the
         * callback (giving the full text, start/end offsets, and groups).
         */

    }, {
        key: "handleClick",
        value: function handleClick() {
            this._opts.onClick(this._processor.pos);
            if (this._opts.followLinks) {
                var pos = this._processor.pos;
                var _iteratorNormalCompletion = true;
                var _didIteratorError = false;
                var _iteratorError = undefined;

                try {
                    for (var _iterator = this._processor.links[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                        var it = _step.value;

                        if (pos >= it.link.start && pos <= it.link.end) {
                            this._opts.onClickLink(it.link);
                            break;
                        }
                    }
                } catch (err) {
                    _didIteratorError = true;
                    _iteratorError = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion && _iterator.return) {
                            _iterator.return();
                        }
                    } finally {
                        if (_didIteratorError) {
                            throw _iteratorError;
                        }
                    }
                }
            }
        }
        /**
         * This event is invoked whenever a change occurs in the editor (any type).
         * @param eventName {string} the name of the event that occurred.
         * @param args {any[]} the dyanamic parameter list passed to this event.
         */

    }, {
        key: "handleEditorChange",
        value: function handleEditorChange(eventName) {
            // debug('handleEditorChange: %s, %o', eventName, args);
            if (eventName === 'selection-change') {
                var range = arguments.length <= 1 ? undefined : arguments[1];
                if (range) {
                    this._processor.range = range;
                    this._processor.pos = range.index;
                }
                // Compute the line that will be involved in highlighting
                this._line = util_section_1.line(this._processor.text, this._processor.pos);
            }
        }
        /**
         * When the user pastes information into the editor, this event is fired
         */

    }, {
        key: "handlePaste",
        value: function handlePaste() {
            this._paste = true;
        }
        /**
         * Invoked with each change of the content text.
         *
         * This will also hold a timer to peform a full scan after N seconds.
         * Using "sections" to format has a trade off where a block format on
         * a boundary of a section can lose formatting where the bottom of the
         * block is seen by the section, but the top is not, so the regex string
         * will not match within the section.  This timer will send a full
         * range of the document to the processor after N seconds (5 seconds by
         * default).  The timer will only occur when changes occur (so it is
         * idle if the keyboard is idle)
         */

    }, {
        key: "handleTextChange",
        value: function handleTextChange(delta, old, source) {
            var _this2 = this;

            delta = old = null;
            if (source === 'user' && this._dirtyCount++ > this._dirtyLimit) {
                this._dirtyIdle = true;
            }
            this._dirtyInline = true;
            if (!this._changing || this._paste) {
                this._changing = true;
                setTimeout(function () {
                    if (_this2._paste) {
                        _this2._processor.refreshFull();
                        _this2._paste = false;
                    } else {
                        _this2._processor.handleChange(_this2._line.start, _this2._line.end);
                    }
                    _this2._opts.onChange(_this2._processor.text);
                    _this2._changing = false;
                }, this._delay);
            }
        }
    }, {
        key: "editor",
        get: function get() {
            return this._editor;
        }
    }, {
        key: "editorKey",
        get: function get() {
            return this.quill.container.id;
        }
    }, {
        key: "fonts",
        get: function get() {
            return this._fonts;
        }
    }, {
        key: "highlights",
        get: function get() {
            return Object.keys(highlights_1.cssHighlights);
        }
    }, {
        key: "opts",
        get: function get() {
            return this._opts;
        }
    }, {
        key: "quill",
        get: function get() {
            return this._quill;
        }
    }]);

    return Markup;
}();

exports.Markup = Markup;